library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity decodemux is
    Port ( decode : in std_logic;
           playStart : in std_logic;
			  recordStart : in std_logic;
			  reset : in std_logic;
           playAddress : in std_logic_vector(18 downto 0);
           recordAddress : in std_logic_vector(18 downto 0);
           playData : out std_logic_vector(31 downto 0);
           recordData : in std_logic_vector(31 downto 0);
           audioRead : in std_logic;
           audioWrite : in std_logic;
           decodeInAddr : in std_logic_vector(18 downto 0);
           decodeOutAddr : in std_logic_vector(18 downto 0);
           decodeInData : out std_logic_vector(31 downto 0);
           decodeOutData : in std_logic_vector(31 downto 0);
           decodeRead : in std_logic;
           decodeWrite : in std_logic;
           inAddress : out std_logic_vector(18 downto 0);
           inData : in std_logic_vector(31 downto 0);
           outData : out std_logic_vector(31 downto 0);
           outAddress : out std_logic_vector(18 downto 0);
           read : out std_logic;
           write : out std_logic);
end decodemux;

architecture Behavioral of decodemux is
	signal decodebuf : bit;
begin

	process (decode, playStart, recordStart, reset)
	begin
		if (reset = '0') then
			decodebuf <= '0';
		elsif (decode = '1') then
			decodebuf <= '1';
		elsif (playStart = '1' or recordStart = '1') then
			decodebuf <= '0';
		end if;
	end process;

	process (decodebuf, inData, decodeWrite, decodeRead, decodeOutData, decodeOutAddr, decodeInAddr, audioWrite,
	         audioRead, recordData, recordAddress, playAddress)
	begin
		if (decodebuf = '1') then
			read <= decodeRead;
			write <= decodeWrite;
			outAddress <= decodeOutAddr;
			inAddress <= decodeInAddr;
			outData <= decodeOutData;
			decodeInData <= inData;
			playData <= inData;
		else
			read <= audioRead;
			write <= audioWrite;
			outAddress <= recordAddress;
			inAddress <= playAddress;
			outData <= recordData;
			decodeInData <= inData;
			playData <= inData;
		end if;
	end process;
end Behavioral;
