library IEEE, SYNOPSYS;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity decoder is
    Port ( read : out std_logic;
           write : out std_logic;
			  inaddr : out std_logic_vector(18 downto 0);
	        outaddr : out std_logic_vector(18 downto 0);
           indata : in std_logic_vector(31 downto 0);  	-- ADPCM encrypted
           outdata : out std_logic_vector(31 downto 0);	-- wave sound file
			  canread : in std_logic;
			  canwrite : in std_logic;
           decode : in std_logic;
			  clk : in std_logic;
			  reset : in std_logic;
			  decodedone : out std_logic;
			  doingstuff : out std_logic);
end decoder;

architecture rtl of decoder is
	----buffer signals---
	signal inputbuffer : std_logic_vector(31 downto 0);
	signal outputbuffer : std_logic_vector(31 downto 0);

	-------types---------
	type stateType is (swait, init, waitread, step1read, postram, incindex, fixindexstep3, 
					step4, step56, step7, waitwrite, stepwrite, upaddr);
	type indextabletype is array (15 downto 0) of integer;
	type stepsizetabletype is array (88 downto 0) of integer;

	------ctrl signals---
	signal state : stateType := swait;
	signal nextstate : stateType := swait;

	-------var signals---	
	signal indextable : indextabletype;
	signal stepsizetable : stepsizetabletype;
	signal sign : bit;
	signal delta : std_logic_vector (3 downto 0);
	signal step : std_logic_vector (15 downto 0);
	signal valpred : integer;
	signal index : integer;
	signal bufferstep : bit_vector(2 downto 0);
	signal localinaddr : std_logic_vector(18 downto 0);
	signal localoutaddr : std_logic_vector(18 downto 0);	
	signal outhalf : bit;


begin

	indextable <= (-1, -1, -1, -1, 2, 4, 6, 8,
	               -1, -1, -1, -1, 2, 4, 6, 8);
	  
	stepsizetable <= (7, 8, 9, 10, 11, 12, 13, 14, 16, 17,
					19, 21, 23, 25, 28, 31, 34, 37, 41, 45,
					50, 55, 60, 66, 73, 80, 88, 97, 107, 118,
					130, 143, 157, 173, 190, 209, 230, 253, 279, 307,
					337, 371, 408, 449, 494, 544, 598, 658, 724, 796,
					876, 963, 1060, 1166, 1282, 1411, 1552, 1707, 1878, 2066, 
					2272, 2499, 2749, 3024, 3327, 3660, 4026, 4428, 4871, 5358, 
					5894, 6484, 7132, 7845, 8630, 9493, 10442, 11487, 12635, 13899,
					15289, 16818, 18500, 20350, 22385, 24623, 27086, 29794, 32767);


	
	process (clk, reset, nextstate)
	begin
		if (reset = '0') then
			state <= swait;
		elsif (clk'EVENT and clk = '1') then
 	   	state <= nextstate;
		end if;	
	end process;


   ------------------next state----------------------------
	controller_ns : process (state)
	-- fills in next state

	begin
		decodedone <= '0';
		case state is 
				when swait => doingstuff <= '1';
						if (decode = '0') then
								nextstate <= swait; 
							else 
						 		nextstate <= init; 
						end if;
				when init => nextstate <= waitread; 
				when waitread => if (canread = '1') then nextstate <= step1read; end if;
				when step1read => nextstate <= postram;
				when postram => nextstate <= incindex;
				when incindex => nextstate <= fixindexstep3;
				when fixindexstep3 => nextstate <= step4;
 		   	when step4 => nextstate <= step56;
			   when step56 => nextstate <= step7;
				when step7 => 		
										if(outhalf = '1') then
											if (bufferstep = "000") then
												nextstate <= waitread;
											else 
												nextstate <= postram;
											end if;
										else
											nextstate <= waitwrite;
										end if;
				when waitwrite => if(canwrite = '1') then nextstate <= stepwrite; end if;
				when stepwrite => nextstate <= upaddr;
				when upaddr =>		if (localoutaddr(18) = '0') then
											nextstate <= swait;
											decodedone <= '1';
										else 
 			   							if (bufferstep = "000") then
												nextstate <= waitread;
											else 
												nextstate <= postram;
											end if;
										end if;
			end case;
	end process;


	---------------output + data path (dependent only on state)-------------
	controller_op : process (state)
		variable stepr3 : integer;
		variable stepr2 : integer;
		variable stepr1 : integer;
		variable stepr0 : integer;
		variable inc1 : integer;
		variable inc2 : integer;
		variable vpdiff : integer;
		variable outop : integer; 
		variable valpredvectortemp : std_logic_vector(15 downto 0);

	begin
			read <= '0';	-- make sure we don't touch memory by default
			write <= '0';	-- ''
			case state is
				when swait => index <= 0;
								  valpred <= 0; 
	
				when init =>  
								inputbuffer <= (others => '0');
								step <= conv_std_logic_vector(stepsizetable(index), 16);			
								bufferstep <= "000"; 
								localinaddr <= (others => '0');
								localoutaddr(17 downto 0) <= (others => '0');
								localoutaddr(18) <= '1'; 
	
	
				when waitread => -- nothing
				when step1read => 
								read <= '1'; 
								inaddr <= localinaddr;
								inputbuffer <= indata;
	
			   when postram =>
								case bufferstep is 
									when "000" => 
														delta <= inputbuffer (31 downto 28); 
														bufferstep <= "001"; 
														localinaddr <= localinaddr + 1;
									when "001" => 
														delta <= inputbuffer (27 downto 24);
														bufferstep <= "010";
									when "010" =>
	 							   					delta <= inputbuffer (23 downto 20);
														bufferstep <= "011";
									when "011" => 
														delta <= inputbuffer (19 downto 16);
														bufferstep <= "100";
									when "100" => 
														delta <= inputbuffer (15 downto 12); 
														bufferstep <= "101"; 
									when "101" => 
														delta <= inputbuffer (11 downto 8);
														bufferstep <= "110";
									when "110" =>
	 							   					delta <= inputbuffer (7 downto 4);
														bufferstep <= "111";
									when "111" => 
														delta <= inputbuffer (3 downto 0);
														bufferstep <= "000";
								end case;
	
				when incindex =>  
								index <= index + indextable(conv_integer(delta));
										
				when fixindexstep3 => 
								if(index < 0) then
									index <= 0;
								elsif (index > 88) then
									index <= 88;
								end if;
								if (delta(3) = '1') then 
									sign <= '1';
								else 
									sign <= '0'; 
								end if;
								delta(3) <= '0';
	
				when step4 => 
								stepr3 := conv_integer("000" & step(15 downto 3));
								stepr2 := conv_integer("00" & step(15 downto 2));
								stepr1 := conv_integer("0" & step(15 downto 1));
								stepr0 := conv_integer(step);
				
								if (delta(0) = '1') then vpdiff := inc2 + stepr0; else vpdiff := inc2; end if;
								if (delta(1) = '1') then inc2 := inc1 + stepr1; else inc2 := inc1; end if;
								if (delta(2) = '1') then inc1 := stepr3 + stepr2; else inc1 := stepr3; end if;
								if (sign = '1') then
									outop := valpred - vpdiff;				
								else
									outop := valpred + vpdiff;
								end if;
								valpred <= outop;
	
				when step56 => if (valpred > 32767) then
										valpred <= 32767;
									elsif (valpred < -32768) then
										valpred <= -32768;
									end if;
									step <= conv_std_logic_vector(stepsizetable(index), 16);			
	
				when step7 => 	
									if(outhalf = '0') then
										valpredvectortemp := conv_std_logic_vector(conv_signed(valpred, 16), 16);
										outputbuffer (31 downto 16) <= valpredvectortemp(15 downto 0);
										outhalf <= '1';
									else
										--(outhalf = '1')
										valpredvectortemp := conv_std_logic_vector(conv_signed(valpred, 16), 16);
										outputbuffer (15 downto 0) <= valpredvectortemp(15 downto 0);
										outhalf <= '0';
									end if;
				when waitwrite => -- nothing
			   when stepwrite => 
									write <= '1'; 
									outdata <= outputbuffer;
									outaddr <= localoutaddr;
	
				when upaddr => 
									localoutaddr <= localoutaddr + 1;
	
			end case;
	end process;
end rtl;


