library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity encodemux is
    Port ( encode : in std_logic;
           playStart : in std_logic;
			  recordStart : in std_logic;
			  reset : in std_logic;
           playAddress : in std_logic_vector(18 downto 0);
           recordAddress : in std_logic_vector(18 downto 0);
           playData : out std_logic_vector(31 downto 0);
           recordData : in std_logic_vector(31 downto 0);
           audioRead : in std_logic;
           audioWrite : in std_logic;
           encodeInAddr : in std_logic_vector(18 downto 0);
           encodeOutAddr : in std_logic_vector(18 downto 0);
           encodeInData : out std_logic_vector(31 downto 0);
           encodeOutData : in std_logic_vector(31 downto 0);
           encodeRead : in std_logic;
           encodeWrite : in std_logic;
           inAddress : out std_logic_vector(18 downto 0);
           inData : in std_logic_vector(31 downto 0);
           outData : out std_logic_vector(31 downto 0);
           outAddress : out std_logic_vector(18 downto 0);
           read : out std_logic;
           write : out std_logic);
end encodemux;

architecture Behavioral of encodemux is
	signal encodebuf : bit;
begin

	process (encode, playStart, recordStart, reset)
	begin
		if (reset = '0') then
			encodebuf <= '0';
		elsif (encode = '1') then
			encodebuf <= '1';
		elsif (playStart = '1' or recordStart = '1') then
			encodebuf <= '0';
		end if;
	end process;

	process (encodebuf, inData, encodeWrite, encodeRead, encodeOutData, encodeOutAddr, encodeInAddr, audioWrite,
	         audioRead, recordData, recordAddress, playAddress)
	begin
		if (encodebuf = '1') then
			read <= encodeRead;
			write <= encodeWrite;
			outAddress <= encodeOutAddr;
			inAddress <= encodeInAddr;
			outData <= encodeOutData;
			encodeInData <= inData;
			playData <= inData;
		else
			read <= audioRead;
			write <= audioWrite;
			outAddress <= recordAddress;
			inAddress <= playAddress;
			outData <= recordData;
			encodeInData <= inData;
			playData <= inData;
		end if;
	end process;
end Behavioral;
